@@ -66,6 +66,7 @@ import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Hashtable;
 import java.util.Iterator;
+import java.util.List;
 import java.util.Map;
 import java.util.Objects;
 import java.util.Queue;
@@ -91,6 +92,7 @@ import com.google.common.collect.ImmutableList;
 import com.google.common.collect.ImmutableMap;
 import com.google.common.collect.ImmutableSet;
 import com.google.common.collect.Iterables;
+import com.google.common.collect.Lists;
 import com.openexchange.ajax.requesthandler.AJAXActionService;
 import com.openexchange.ajax.requesthandler.AJAXActionServiceFactory;
 import com.openexchange.config.ForcedReloadable;
@@ -109,9 +111,11 @@ import com.openexchange.util.activator.Property;
 import com.openexchange.util.activator.Provides;
 import com.openexchange.util.activator.RegistrationException;
 import com.openexchange.util.activator.Service;
+import com.openexchange.util.activator.ServiceClass;
 import com.openexchange.util.activator.ServiceDependencies;
 import com.openexchange.util.activator.ServiceDependencyResolver;
 import com.openexchange.util.activator.ServiceFactory;
+import com.openexchange.util.activator.ServiceRef;
 import com.openexchange.util.activator.ServiceRegistrationHandler;
 import com.openexchange.util.activator.ServiceWithProperties;
 import com.openexchange.util.activator.SetOfServices;
@@ -250,6 +254,18 @@ public enum RegistrationTools {
         return null;
     }
 
+    public static final @Nullable ServiceClass serviceClassAnnotation(final @Nullable Annotation[] annotations) {
+        if (annotations == null) {
+            return null;
+        }
+        for (final Annotation a : annotations) {
+            if (a instanceof ServiceClass) {
+                return (ServiceClass) a;
+            }
+        }
+        return null;
+    }
+    
     public static final <T> ImmutableList<Constructor<T>> serviceConstructors(final @Nullable Constructor<T>[] constructors) {
         if (constructors == null) {
             return ImmutableList.of();
@@ -263,28 +279,34 @@ public enum RegistrationTools {
         return b.build();
     }
 
-
     public static final Optional<Object[]> pickDependencies(final Method method,
-        Map<ServiceDependency<?>, Object> dependencies,
+        Map<MandatoryServiceDependency<?>, Object> mandatoryDependencies,
+        List<OptionalServiceDependency<?>> optionalDependencies,
         Map<Class<?>, ServiceSet<?>> serviceSetMap,
         Map<Class<?>, ServiceListing<?>> serviceListingMap,
         Dictionary<String, ?> properties) {
         return pickDependencies(notNull(method.getParameterTypes()), notNull(method.getParameterAnnotations()),
-            dependencies, serviceSetMap, serviceListingMap, properties);
+            mandatoryDependencies, optionalDependencies,
+            serviceSetMap, serviceListingMap,
+            properties);
     }
     
     public static final Optional<Object[]> pickDependencies(final Constructor<?> ctor,
-        Map<ServiceDependency<?>, Object> dependencies,
+        Map<MandatoryServiceDependency<?>, Object> mandatoryDependencies,
+        List<OptionalServiceDependency<?>> optionalDependencies,
         Map<Class<?>, ServiceSet<?>> serviceSetMap,
         Map<Class<?>, ServiceListing<?>> serviceListingMap,
         Dictionary<String, ?> properties) {
         return pickDependencies(notNull(ctor.getParameterTypes()), notNull(ctor.getParameterAnnotations()),
-            dependencies, serviceSetMap, serviceListingMap, properties);
+            mandatoryDependencies, optionalDependencies,
+            serviceSetMap, serviceListingMap,
+            properties);
     }
 
     private static final Optional<Object[]> pickDependencies(final Class<?>[] types,
         final Annotation[][] annotationArrays,
-        Map<ServiceDependency<?>, Object> dependencies,
+        Map<MandatoryServiceDependency<?>, Object> mandatoryDependencies,
+        List<OptionalServiceDependency<?>> optionalDependencies,
         Map<Class<?>, ServiceSet<?>> serviceSetMap,
         Map<Class<?>, ServiceListing<?>> serviceListingMap,
         Dictionary<String, ?> properties) {
@@ -305,16 +327,33 @@ public enum RegistrationTools {
                     }
                 }
             } else if (type.equals(ServiceListing.class)) {
-                    final SetOfServices a = setOfServicesAnnotation(annotations);
-                    if (a != null) {
-                        final Class<?> serviceListingType = a.value();
-                        final ServiceListing<?> serviceListing = serviceListingMap.get(serviceListingType);
-                        if (serviceListing == null) {
-                            return Optional.absent();
-                        } else {
-                            values[i] = serviceListing;
+                final SetOfServices a = setOfServicesAnnotation(annotations);
+                if (a != null) {
+                    final Class<?> serviceListingType = a.value();
+                    final ServiceListing<?> serviceListing = serviceListingMap.get(serviceListingType);
+                    if (serviceListing == null) {
+                        return Optional.absent();
+                    } else {
+                        values[i] = serviceListing;
+                    }
+                }
+            } else if (type.equals(ServiceRef.class)) {
+                final ServiceClass a = RegistrationTools.serviceClassAnnotation(annotations);
+                if (a != null) {
+                    final Class<?> dependencyType = a.value();
+                    boolean matched = false;
+                    final Iterator<OptionalServiceDependency<?>> iter = optionalDependencies.iterator();
+                    while ((! matched) && iter.hasNext()) {
+                        final OptionalServiceDependency<?> e = iter.next();
+                        if (e.matchesDependency(dependencyType, annotations)) {
+                            values[i] = e.serviceRef();
+                            matched = true;
                         }
                     }
+                    if (! matched) {
+                        return Optional.absent();
+                    }
+                }
             } else if (type.equals(Dictionary.class)) {
                 values[i] = properties;
             } else {
@@ -327,12 +366,11 @@ public enum RegistrationTools {
                         return Optional.absent();
                     }
                 } else {
-                    final @Nullable Service serviceAnnotation = serviceAnnotation(annotations);
                     boolean matched = false;
-                    final Iterator<Map.Entry<ServiceDependency<?>, Object>> iter = dependencies.entrySet().iterator();
+                    final Iterator<Map.Entry<MandatoryServiceDependency<?>, Object>> iter = mandatoryDependencies.entrySet().iterator();
                     while ((! matched) && iter.hasNext()) {
-                        final Map.Entry<ServiceDependency<?>, Object> e = iter.next();
-                        if (e.getKey().matchesDependency(type, serviceAnnotation)) {
+                        final Map.Entry<MandatoryServiceDependency<?>, Object> e = iter.next();
+                        if (e.getKey().matchesDependency(type, annotations)) {
                             values[i] = e.getValue();
                             matched = true;
                         }
@@ -353,16 +391,20 @@ public enum RegistrationTools {
     };
     
     public static final @Nullable Filter generateServiceFilter(final BundleContext context,
-        final Collection<ServiceDependency<?>> serviceDependencies) throws InvalidSyntaxException {
-        if (serviceDependencies.isEmpty()) {
+        final Collection<MandatoryServiceDependency<?>> mandatoryServiceDependencies,
+        final Collection<OptionalServiceDependency<?>> optionalServiceDependencies) throws InvalidSyntaxException {
+        
+        final List<ServiceDependencyTemplate<?>> dependencies = Lists.newArrayList(Iterables.concat(mandatoryServiceDependencies, optionalServiceDependencies));
+        
+        if (dependencies.isEmpty()) {
             return null;
         }
         
         final String filter;
-        if (serviceDependencies.size() == 1) {
-            filter = TO_OSGI_FILTER.apply(serviceDependencies.iterator().next());
+        if (dependencies.size() == 1) {
+            filter = TO_OSGI_FILTER.apply(dependencies.get(0));
         } else {
-            filter = String.format("(|%s)", FluentIterable.from(serviceDependencies)
+            filter = String.format("(|%s)", FluentIterable.from(dependencies)
                 .transform(TO_OSGI_FILTER)
                 .join(Joiner.on(""))
             );
@@ -578,7 +620,9 @@ public enum RegistrationTools {
         if (serviceInstance instanceof DependencyProvider) {
             service = new DependencyProviderServiceWithDependencies<S, T>(serviceInstance, optServiceClass);
         } else {
-            service = new ServiceInstanceServiceWithDependencies<S, T>(serviceInstance, optServiceClass, ImmutableSet.<ServiceDependency<?>>of());
+            service = new ServiceInstanceServiceWithDependencies<S, T>(
+                serviceInstance, optServiceClass, null, null
+            );
         }
         service.registerService(bundleContext, registry, properties, serviceRegistrationHandler);
     }
@@ -630,7 +674,8 @@ public enum RegistrationTools {
         final String path) {
         final ImmutableSet<Class<?>> serviceClasses = ImmutableSet.<Class<?>>of(HttpServlet.class);
 
-        final ImmutableSet<ServiceDependency<?>> dependencies;
+        final ImmutableSet<MandatoryServiceDependency<?>> mandatoryDependencies;
+        final ImmutableSet<OptionalServiceDependency<?>> optionalDependencies;
         final ImmutableSet<Class<?>> serviceSetClasses;
         final ImmutableSet<Class<?>> serviceListingClasses;
         {
@@ -640,7 +685,8 @@ public enum RegistrationTools {
             } else {
                 serviceDependencies = ImmutableServiceDependencies.empty();
             }
-            dependencies = ServletServiceRegistererTemplate.withServletDependencies(serviceDependencies.serviceDependencies(), path);
+            mandatoryDependencies = ServletServiceRegistererTemplate.withMandatoryServletDependencies(serviceDependencies.mandatoryServiceDependencies(), path);
+            optionalDependencies = ServletServiceRegistererTemplate.withOptionalServletDependencies(serviceDependencies.optionalServiceDependencies(), path);
             serviceSetClasses = serviceDependencies.serviceSetClasses();
             serviceListingClasses = serviceDependencies.serviceListingClasses();
         }
@@ -655,7 +701,7 @@ public enum RegistrationTools {
             serviceListingClasses
         );
         
-        final ImmutableSet<String> propertiesOfInterest = collectPropertiesOfInterest(dependencies);
+        final ImmutableSet<String> propertiesOfInterest = collectPropertiesOfInterest(mandatoryDependencies, optionalDependencies);
         
         final ServletServiceRegistererTemplate<T> registerer =
             new ServletServiceRegistererTemplate<T>(path,
@@ -663,7 +709,8 @@ public enum RegistrationTools {
                 serviceClasses,
                 className(serviceInstance),
                 properties,
-                dependencies,
+                mandatoryDependencies,
+                optionalDependencies,
                 serviceSets,
                 serviceListings,
                 propertiesOfInterest) {
@@ -678,7 +725,8 @@ public enum RegistrationTools {
         };
         RegistrationTools.<HttpServlet, T>trackAndOpen(
             bundleContext,
-            dependencies,
+            mandatoryDependencies,
+            optionalDependencies,
             registerer,
             className(serviceInstance)
         );
@@ -703,7 +751,8 @@ public enum RegistrationTools {
         final MBeanRegistrar<S, T> r = new MBeanRegistrar<S, T>(bundleContext, registry, objectName, mbeanInterface, mbean);
         trackAndOpen(
             bundleContext,
-            ImmutableSet.<ServiceDependency<?>>of(ServiceClassDependency.ofClass(ManagementService.class)),
+            ImmutableSet.<MandatoryServiceDependency<?>>of(new MandatoryServiceDependency<>(ManagementService.class)),
+            ImmutableSet.<OptionalServiceDependency<?>>of(),
             r,
             notNull(objectName.toString())
         );
@@ -741,17 +790,21 @@ public enum RegistrationTools {
         final ActivatorRegistry registry,
         final Class<F> filterClass,
         final int ranking) {
+        final ServiceTrackerCustomizer<DispatcherPrefixService, DispatcherPrefixService> customizer = new DispatcherPrefixServiceTrackerCustomizer(bundleContext) {
+            @Override
+            protected void onDispatcherPrefixServiceAvailable(final DispatcherPrefixService dispatcherPrefixService) throws Exception {
+                final Hashtable<String, Object> properties = new Hashtable<>(2);
+                properties.put(org.osgi.framework.Constants.SERVICE_RANKING, Integer.valueOf(ranking));
+                properties.put(com.openexchange.servlet.Constants.FILTER_PATHS, dispatcherPrefixService.getPrefix() + "login");
+                registerServiceClass(bundleContext, registry, javax.servlet.Filter.class, filterClass, properties, null);
+            }
+        };
         final ServiceTracker<DispatcherPrefixService, DispatcherPrefixService> tracker = new ServiceTracker<DispatcherPrefixService, DispatcherPrefixService>(bundleContext,
-            DispatcherPrefixService.class, new DispatcherPrefixServiceTrackerCustomizer(bundleContext) {
-                @Override
-                protected void onDispatcherPrefixServiceAvailable(final DispatcherPrefixService dispatcherPrefixService) throws Exception {
-                    final Hashtable<String, Object> properties = new Hashtable<>(2);
-                    properties.put(org.osgi.framework.Constants.SERVICE_RANKING, Integer.valueOf(ranking));
-                    properties.put(com.openexchange.servlet.Constants.FILTER_PATHS, dispatcherPrefixService.getPrefix() + "login");
-                    registerServiceClass(bundleContext, registry, javax.servlet.Filter.class, filterClass, properties, null);
-                }
-            });
+            DispatcherPrefixService.class, customizer);
         tracker.open();
+        if (customizer instanceof ExtendedServiceTrackerCustomizer) {
+            ((ExtendedServiceTrackerCustomizer<DispatcherPrefixService, DispatcherPrefixService>) customizer).onOpened();
+        }
         registry.add(tracker);
     }
     
@@ -759,17 +812,21 @@ public enum RegistrationTools {
         final ActivatorRegistry registry,
         final F filter,
         final int ranking) {
+        final ServiceTrackerCustomizer<DispatcherPrefixService, DispatcherPrefixService> customizer = new DispatcherPrefixServiceTrackerCustomizer(bundleContext) {
+            @Override
+            protected void onDispatcherPrefixServiceAvailable(final DispatcherPrefixService dispatcherPrefixService) throws Exception {
+                final Hashtable<String, Object> properties = new Hashtable<>(2);
+                properties.put(org.osgi.framework.Constants.SERVICE_RANKING, Integer.valueOf(ranking));
+                properties.put(com.openexchange.servlet.Constants.FILTER_PATHS, dispatcherPrefixService.getPrefix() + "login");
+                RegistrationTools.registerServiceInstance(bundleContext, registry, javax.servlet.Filter.class, filter, properties, null);
+            }
+        };
         final ServiceTracker<DispatcherPrefixService, DispatcherPrefixService> tracker = new ServiceTracker<DispatcherPrefixService, DispatcherPrefixService>(bundleContext,
-            DispatcherPrefixService.class, new DispatcherPrefixServiceTrackerCustomizer(bundleContext) {
-                @Override
-                protected void onDispatcherPrefixServiceAvailable(final DispatcherPrefixService dispatcherPrefixService) throws Exception {
-                    final Hashtable<String, Object> properties = new Hashtable<>(2);
-                    properties.put(org.osgi.framework.Constants.SERVICE_RANKING, Integer.valueOf(ranking));
-                    properties.put(com.openexchange.servlet.Constants.FILTER_PATHS, dispatcherPrefixService.getPrefix() + "login");
-                    RegistrationTools.registerServiceInstance(bundleContext, registry, javax.servlet.Filter.class, filter, properties, null);
-                }
-            });
+            DispatcherPrefixService.class, customizer);
         tracker.open();
+        if (customizer instanceof ExtendedServiceTrackerCustomizer) {
+            ((ExtendedServiceTrackerCustomizer<DispatcherPrefixService, DispatcherPrefixService>) customizer).onOpened();
+        }
         registry.add(tracker);
     }
     
@@ -841,7 +898,7 @@ public enum RegistrationTools {
         final ActivatorRegistry registry,
         final Class<S> serviceClass) {
         final RankingAwareNearRegistryServiceTracker<S> tracker = new RankingAwareNearRegistryServiceTracker<S>(bundleContext, serviceClass);
-        tracker.open();
+        tracker.open();        
         registry.add(tracker);
         return tracker;
     }
@@ -863,9 +920,12 @@ public enum RegistrationTools {
         final ActivatorRegistry registry,
         final Class<S> serviceClass,
         final SimpleRegistryListener<S> serviceSet) {
-        final ServiceTracker<S, S> tracker = new ServiceTracker<S, S>(bundleContext, serviceClass,
-            new SimpleRegistryListenerTrackerCustomizer<S>(serviceSet, bundleContext));
+        final ServiceTrackerCustomizer<S, S> customizer = new SimpleRegistryListenerTrackerCustomizer<S>(serviceSet, bundleContext);
+        final ServiceTracker<S, S> tracker = new ServiceTracker<S, S>(bundleContext, serviceClass, customizer);
         tracker.open();
+        if (customizer instanceof ExtendedServiceTrackerCustomizer) {
+            ((ExtendedServiceTrackerCustomizer<S, S>) customizer).onOpened();
+        }
         registry.add(tracker);
     }
 
@@ -875,16 +935,20 @@ public enum RegistrationTools {
         final ServiceTrackerCustomizer<S, S> customizer) {
         final ServiceTracker<S, S> tracker = new ServiceTracker<S, S>(bundleContext, serviceClass, customizer);
         tracker.open();
+        if (customizer instanceof ExtendedServiceTrackerCustomizer) {
+            ((ExtendedServiceTrackerCustomizer<S, S>) customizer).onOpened();
+        }
         registry.add(tracker);
     }    
     
     public static <S, T extends S> ServiceTracker<S, T> trackAndOpen(final BundleContext bundleContext,
-        final ImmutableSet<ServiceDependency<?>> dependencies,
+        final ImmutableSet<MandatoryServiceDependency<?>> mandatoryDependencies,
+        final ImmutableSet<OptionalServiceDependency<?>> optionalDependencies,
         final ServiceTrackerCustomizer<S, T> registerer,
         final String serviceInstanceName) {
         final ServiceTracker<S, T> serviceTracker;
         try {
-            final Filter filter = generateServiceFilter(bundleContext, dependencies);
+            final Filter filter = generateServiceFilter(bundleContext, mandatoryDependencies, optionalDependencies);
             if (filter != null) {
                 serviceTracker = new ServiceTracker<S, T>(bundleContext, filter, registerer);
             } else {
@@ -914,6 +978,11 @@ public enum RegistrationTools {
             LOG.error(msg, t);
             throw new RegistrationException(msg, t);
         }
+        
+        if (registerer instanceof ExtendedServiceTrackerCustomizer) {
+            ((ExtendedServiceTrackerCustomizer<S, T>) registerer).onOpened();
+        }
+        
         return serviceTracker;
     }
     
@@ -993,10 +1062,13 @@ public enum RegistrationTools {
         return null;
     }
 
-    public static ImmutableSet<String> collectPropertiesOfInterest(final Iterable<ServiceDependency<?>> serviceDependencies) {
+    @SafeVarargs
+    public static ImmutableSet<String> collectPropertiesOfInterest(final Collection<? extends ServiceDependency<?>>... serviceDependencies) {
         final Set<String> set = new HashSet<>();
-        for (final ServiceDependency<?> d : serviceDependencies) {
-            set.addAll(d.propertiesOfInterest());
+        for (final Collection<? extends ServiceDependency<?>> collection : serviceDependencies) {
+            for (final ServiceDependency<?> d : collection) {
+                set.addAll(d.propertiesOfInterest());
+            }
         }
         return ImmutableSet.copyOf(set);
     }
